@@ -1,6 +1,3 @@
-# $File: //member/autrijus/PAR/lib/PAR/Filter/PodStrip.pm $ $Author: autrijus $
-# $Revision: #5 $ $Change: 10218 $ $DateTime: 2004/02/26 23:44:36 $
-
 package PAR::Filter::PodStrip;
 
 use strict;
@@ -38,7 +35,7 @@ sub apply {
     $$ref =~ s{(
 	(.*?\n)
 	(?:=(?:head\d|pod|begin|item|over|for|back|end)\b
-	.*?\n)?
+    .*?\n)
 	(?:=cut[\t ]*[\r\n]*?|\Z)
 	(\r?\n)?
     )}{
